from tests.test_configs.config_helper import (
    load_and_validate,
    load_and_validate_fails,
    load_config,
)
from pencil_pusher.configs import config_manager


def test_valid():
    """
    Tests that a valid config file contains the required
    parameter titles
    """
    load_config()
    load_and_validate()


def test_not_defined():
    """
    Tests that a this config is optional
    """
    load_config(exclude=["python"])
    load_and_validate()


def test_invalid_type():
    """
    """
    load_config(python=3)
    load_and_validate_fails()


def test_invalid_subtype_init():
    """
    """
    load_config(
        python={
            "include_init_files": "no",
            "include_main_file": False,
            "module": "mymodule",
        }
    )
    load_and_validate_fails()


def test_invalid_subtype_main():
    """
    """
    load_config(
        python={
            "include_init_files": False,
            "include_main_file": "no",
            "module": "mymodule",
        }
    )
    load_and_validate_fails()


def test_invalid_subtype_module():
    """
    """
    load_config(
        python={
            "include_init_files": False,
            "include_main_file": False,
            "module": 6,
        }
    )
    load_and_validate_fails()


def test_missing_subtype_main():
    """
    """
    load_config(python={"include_init_files": False, "module": "mymod"})
    load_and_validate()


def test_missing_subtype_init():
    """
    """
    load_config(python={"include_main_file": False, "module": "mymod"})
    load_and_validate()


def test_missing_subtype_module():
    """
    """
    load_config(
        python={"include_main_file": False, "include_init_files": False}
    )
    load_and_validate()


def test_get_python():
    """
    """
    expected = {
        "include_init_files": True,
        "include_main_file": True,
        "module": "mymod",
    }

    load_config(python=expected)
    load_and_validate()

    response = config_manager.get("python")
    assert response == expected


def test_get_python_default():
    """
    """
    load_config(exclude=["python"])
    load_and_validate()

    response = config_manager.get("python")
    assert response is None



from pencil_pusher.documenter import Documenter
import os


def test_gen_local():
    """
    Tests generating a doc with a
    local file
    """
    Documenter.generate("pencil_pusher.documenter", "test")
    assert os.path.isfile("./test.md")
    os.remove("./test.md")


def test_toc():
    """
    Tests that toc is generated correctly
    """

    test_files = [
        "pencil_pusher.configs.base.py",
        "pencil_pusher.configs.manager.py",
        "pencil_pusher.configs.python_lang.py",
        "pencil_pusher.documenter.py",
        "pencil_pusher.input.py",
    ]

    test_file_map = {
        ".".join(i.split(".")[:-1]): ".".join(i.split(".")[:-1])
        for i in test_files
    }
    toc_tree = Documenter.build_toc(test_files)

    assert toc_tree == {
        "pencil_pusher": {
            "configs": {"base": {}, "manager": {}, "python_lang": {}},
            "documenter": {},
            "input": {},
        }
    }

    Documenter.generate_sidebar(toc_tree, test_file_map)

    os.remove("./_Sidebar.md")



import json
from typing import Any, Dict, List, Tuple
from sd_utils.plugins.plugin_manager import PluginManager


class ConfigManager(PluginManager):
    """
    Manager for all config variables. Provides interface
    for loading, validating and getting configs from a
    file.
    """

    VALIDATE = "validate"
    """
    constant for validate action
    """

    GET = "get"
    """
    constant for get action.
    """

    PROCESS_LANG_FILE = "process_lang_file"
    """
    constant for process lang file action
    """

    def __init__(self):
        self._config_file_contents = None
        self._languages = {}
        super().__init__()

    def validate(self):
        """
        Validates all the configs in the loaded config file

        Raises:
            ValueError: if the config file is not loaded
        """
        if self._config_file_contents is None:
            raise ValueError("Config file was not loaded")

        self.iterate_all(
            {
                "type": ConfigManager.VALIDATE,
                "contents": self._config_file_contents,
            },
        )

    def get(self, name: str) -> Any:
        """
        Gets a config defined by `name`

        Raises:
            ValueError: if the config file is not loaded

        Args:
            name (str): the name of the config being retrieved

        Returns:
            Any: the value of the config
        """
        if self._config_file_contents is None:
            raise ValueError("Config file was not loaded")

        return self.run(
            name,
            on_find_params={
                "type": ConfigManager.GET,
                "contents": self._config_file_contents,
            },
        )

    def has(self, name: str) -> bool:
        """
        Returns true if a given config is defined (false otherwise)

        Args:
            name (str): the name of the config being searched for

        Returns:
            bool: True if the config exists, false otherwise
        """

    def process_lang_file(self, file_path: str) -> Tuple[bool, str]:
        """
        processes a language file based on defined configurations

        Args:
            file_path (str): the path of the config file to load.
                (must be a local location)

        Returns:
            Tuple[bool, str]: the first value is if the file should be included,
                the second value is the new name of the file if it should be
                included.
        """
        ext = file_path[file_path.index(".") + 1 :]
        lang_name = ""
        for lang, exts in self._languages.items():
            for dext in exts:
                if ext == dext:
                    lang_name = lang
                    break
        return self.run(
            lang_name,
            on_find_params={
                "type": ConfigManager.PROCESS_LANG_FILE,
                "contents": self._config_file_contents,
                "file_path": file_path,
            },
        )

    def load_config_file(self, file_path: str):
        """
        Loads a given config file (must be a local
        file location)

        Args:
            file_path (str): path of the config file
                (must be a local location)
        """
        with open(file_path, "r") as fp:
            self._config_file_contents = json.load(fp)

    def get_on_search_params(self, name: str, **kwargs) -> Any:
        return {"name": name, **kwargs}

    def get_on_find_params(self, name: str, **kwargs) -> Any:
        return {"name": name, **kwargs}

    def get_on_register_params(self, name: str, **kwargs) -> Any:
        return {"manager": self}

    def add_lang(self, lang_name: str, extensions: List[str]):
        """
        Adds a language and its relevant file extensions to
        this config manager.
        """
        self._languages[lang_name] = extensions

    def get_langs(self) -> Dict[str, List[str]]:
        """
        Returns a dictionary which maps each language to
        its extensions

        Returns:
            Dict[str, List[str]]: a dict of languages and their
                file extensions
        """
        return self._languages



from pencil_pusher.file_util import FileUtil


def test_query_directory():
    """
    Tests that source files are queried properly
    """
    result = FileUtil.query_directory("pencil_pusher", [".py"])

    count = 0
    for r in result:
        if "pencil_pusher/__init__.py" in r:
            count += 1
        if "pencil_pusher/__main__.py" in r:
            count += 1

    assert count == 2



from tests.test_configs.config_helper import (
    load_and_validate,
    load_and_validate_fails,
    load_config,
)
from pencil_pusher.configs import config_manager


def test_valid():
    """
    Tests that a valid config file contains the required
    parameter title_prefix
    """
    load_config()
    load_and_validate()


def test_invalid_type():
    """
    """
    load_config(title_prefix=3)
    load_and_validate_fails()


def test_not_defined():
    """
    """
    load_config(exclude=["title_prefix"])
    load_and_validate()


def test_get_title_prefix():
    """
    """
    expected = "myprefix"

    load_config(title_prefix=expected)
    load_and_validate()

    response = config_manager.get("title_prefix")
    assert response == expected


def test_get_title_prefix_default():
    """
    """
    load_config(exclude=["title_prefix"])
    load_and_validate()

    response = config_manager.get("title_prefix")
    assert response == ""



import os
from tests.test_configs.config_helper import load_and_validate, load_config
from pencil_pusher.file_util import FileUtil
from pencil_pusher.documenter import Documenter
from pencil_pusher.github_api import GithubAPI
from pencil_pusher.repo_manager import RepoManager


def test_repo_manager_simple_config():
    """
    Tests running the repo manager with the
    most basic config that can be provided
    """
    load_config(
        sources=["biit_server/"], title_prefix="", title_suffix="", python={}
    )
    load_and_validate()

    Documenter.install()

    owner = "biit-407"
    repo = "biit-server"
    ghapi = GithubAPI(os.environ["GH_PAT"])
    rm = RepoManager(owner, repo, ghapi)

    rm.setup()

    rm.install()

    rm.document(sources=["biit_server/"], extensions=["py"])
    # no publish, just testing

    files_to_check = FileUtil.query_directory(
        "../biit-server/biit_server/", [".py"]
    )
    files_found = FileUtil.query_directory("../biit-server-wiki/", [".md"])

    for fc in files_to_check:
        fc_name = fc[fc.index("biit_server") : -3]
        fc_name = fc_name.replace("/", ".")
        found = False
        for ff in files_found:
            if fc_name in ff:
                found = True
                break
        if fc_name.endswith("__init__") or fc_name.endswith("__main__"):
            continue
        assert found, f"could not find [{fc_name}]"

    rm.cleanup()



from pencil_pusher.github_api import GithubAPI
import os
import shutil

REPO = "ghapd"
OWNER = "stephend017"
ROOT_DIR = os.path.dirname(os.path.abspath("./setup.py"))


def test_get_public_file():
    """
    Tests that a public file can be successfully queried from
    a public repository
    """
    file_path = "action.yml"

    contents = GithubAPI.get_public_file(
        OWNER, REPO, file_path, os.environ["GH_PAT"]
    )

    assert 'name: "pencil_pusher"' in contents


def test_clone_repo():
    """
    Tests that a repo can be cloned successfully to a
    local location
    """
    ghapi = GithubAPI(os.environ["GH_PAT"])
    ghapi.clone_repo(OWNER, REPO, "example-repo")
    path = os.path.join(ROOT_DIR, "example-repo")

    assert os.path.exists(path)
    shutil.rmtree(path)


def test_clone_repo_external():
    """
    Tests that a repo can be cloned successfully to a
    local location that is outside of the executing directory

    This is needed so the wiki repo and the source repo can be
    used in tandem without having to deal with git potentially
    interacting with both of them
    """
    ghapi = GithubAPI(os.environ["GH_PAT"])
    ghapi.clone_repo(OWNER, REPO, "../example-repo")
    path = os.path.join(ROOT_DIR, "../example-repo")

    assert os.path.exists(path)
    shutil.rmtree(path)


def test_full_send():
    """
    Tests that a repo can be automatically staged, committed and
    pushed
    """
    ghapi = GithubAPI(os.environ["GH_PAT"])
    ghapi.clone_repo(OWNER, "pencil-pusher.wiki", "../example-repo")
    path = os.path.join(ROOT_DIR, "../example-repo")

    # do a change
    with open(os.path.join(path, "_testing.md"), "a+") as fp:
        fp.write("* testing string \n\n")

    ghapi.full_update(OWNER, "ghapd.wiki", path)

    shutil.rmtree(path)


def test_full_send_relative_path():
    """
    Tests that a repo can be automatically staged, committed and
    pushed from a relative path
    """
    ghapi = GithubAPI(os.environ["GH_PAT"])
    ghapi.clone_repo(OWNER, "pencil-pusher.wiki", "../example-repo")

    # do a change
    with open(os.path.join("../example-repo", "_testing.md"), "a+") as fp:
        fp.write("* testing string from relative \n\n")

    ghapi.full_update(OWNER, "pencil-pusher.wiki", "../example-repo")

    shutil.rmtree("../example-repo")



from typing import Any
from pencil_pusher.configs.manager import ConfigManager
from pencil_pusher.configs.base import ConfigParamBase
from pencil_pusher.configs import config_manager


@config_manager.register("sidebar")
class SidebarConfigParam(ConfigParamBase):
    """
    Processes the sidebar config parameter

    Note: validation only validates config name
        and config type (string array)
    """

    def on_iterate(self, data: Any):
        """
        This is the validate function
        """
        if data["type"] == ConfigManager.VALIDATE:
            self._validate(data["contents"])
            self.set_defined()

    def _validate(self, contents: dict) -> bool:
        """
        Validates the parameter is included in the
        config file since it is a required field
        """
        if "sidebar" not in contents:
            # set default value
            contents["sidebar"] = False
            return True

        if not isinstance(contents["sidebar"], bool):
            raise ValueError("sidebar config parameter is not of type bool")

        return True



"""
Wrapper class for managing complex github commands needed to be
executed for this action
"""
from pencil_pusher.process_util import ProcessUtil
from github import Github
from github.ContentFile import ContentFile
from github.Repository import Repository


class GithubAPI:
    def __init__(self, personal_access_token: str = ""):
        """
        Creates a new GithubAPI wrapper object for computing
        complex github operations

        Args:
            personal_access_token (str): the github token to use
                for authentication.
        """
        self._has_token = personal_access_token != ""
        self._pat = personal_access_token
        if self._has_token:
            self._Github = Github(personal_access_token)
        else:
            self._Github = Github()

    def clone_repo(self, owner: str, repo: str, local_path: str = ""):
        """
        Clones a given github repo to a given local file path

        Args:
            owner (str): the owner of the repo
            repo (str): the name of the repo
            local_path (str): the local path to store this repo
        """
        url = f"https://github.com/{owner}/{repo}.git"
        if self._has_token:
            url = f"https://{owner}:{self._pat}@github.com/{owner}/{repo}.git"

        ProcessUtil.execute(["git", "clone", f"{url}", f"{local_path}"])

    def full_update(self, owner: str, repo: str, path: str):
        """
        stages, commits and pushes all changes to a git repo
        in the given directory.

        NOTE: will not create a commit if no files were changed

        Args:
            owner (str): the owner of the repo
            repo (str): the name of the repo
            path (str): the location of the repo
        """
        ProcessUtil.execute(
            [
                "git",
                "remote",
                "set-url",
                "origin",
                f"https://{owner}:{self._pat}@github.com/{owner}/{repo}.git",
            ],
            cwd=path,
        )

        ProcessUtil.execute(
            ["git", "config", "user.name", "pencil-pusher"], cwd=path
        )

        ProcessUtil.execute(["git", "config", "user.email", "<>"], cwd=path)

        ProcessUtil.execute(["git", "add", "."], cwd=path)

        returncode: int = ProcessUtil.execute(
            ["git", "commit", "-m", '"Auto commit by pencil-pusher"'], cwd=path
        )
        if returncode == 1:
            print("all docs are up to date")
            return

        ProcessUtil.execute(["git", "push"], cwd=path)

    @staticmethod
    def get_public_file(
        owner: str, repo: str, file_path: str, personal_access_token: str = ""
    ) -> str:
        """
        Gets a file from a public repo

        Args:
            owner (str): the owner the repo being queried
            repo (str): the name of the repo being queried
            file_path (str): the relative path of the file
                in the repo being queried
            personal_access_token (str): the github token used
                for authentication, required for private repos
        """
        g = (
            Github()
            if personal_access_token == ""
            else Github(personal_access_token)
        )
        repo: Repository = g.get_repo(f"{owner}/{repo}")
        response: ContentFile = repo.get_contents(file_path)
        return str(response.decoded_content, "utf-8")

    def _require_token(self):
        """
        Simple function that raises a Value error if a
        token is not defined but is required for the operation
        """
        if not self._has_token:
            raise ValueError(
                "The selected operation requires that a token be provided"
            )



"""
File for processing and validating inputs
"""
from pencil_pusher.github_api import GithubAPI
from os import environ
from typing import Any, Dict, Tuple
import yaml
import time


class InputDefinition:
    def __init__(self, name: str, description: str, required: bool = False):
        """"""
        self._name = name
        self._description = description
        self._required = required

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def required(self) -> bool:
        return self._required


class InputManager:
    def __init__(self):
        """
        Manager class for input provided in the action file
        """
        self._input_definition: Dict[str, InputDefinition] = {}
        self._provided_inputs = {
            "GITHUB_OWNER_NAME": "",
            "GITHUB_REPOSITORY_NAME": "",
        }

    def define(self, personal_access_token: str = ""):
        """
        Defines the expected inputs from the action file
        located in the pencil-pusher repo

        Args:
            personal_access_token (str): the github token
                to use for authentication or to increase
                the rate limit
        """
        InputManager._sync_action_file(personal_access_token)
        # user defined inputs
        with open("./action.yml", "r") as fp:
            definition = yaml.safe_load(fp)
            for input_name, input_definition in definition["inputs"].items():
                self._input_definition[input_name] = InputDefinition(
                    input_name,
                    input_definition["description"],
                    input_definition["required"],
                )

        # provided inputs
        repo_slug = environ["GITHUB_REPOSITORY"]
        self._provided_inputs["GITHUB_OWNER_NAME"] = repo_slug.split("/")[0]
        self._provided_inputs["GITHUB_REPOSITORY_NAME"] = repo_slug.split("/")[
            1
        ]

    def validate(self) -> Tuple[bool, str]:
        """
        Validates that all inputs in the given action
        file match those of the defined action file

        Returns:
            Tuple[bool, str]: a flag if validation succeeded
                and a message why it either succeeded or failed
        """
        for key, value in self._input_definition.items():
            environ_key: str = InputManager._environ_key(key)
            if environ_key not in environ:
                return False, f"Undefined input [{key}] provided"
            if value.required and (
                environ[environ_key] is None or environ[environ_key] == ""
            ):
                return (
                    False,
                    f"Required Input [{key}] has illegal value [{environ[environ_key]}]",
                )

        return True, "Successfully validated all inputs"

    def get(self, name: str) -> Any:
        """
        Gets an input by name

        Args:
            name (str): the name of the input variable
        """
        environ_input_key: str = InputManager._environ_key(name)
        environ_provided_key: str = InputManager._environ_provided_key(name)
        if (
            environ_input_key not in environ
            and environ_provided_key not in self._provided_inputs
        ):
            raise ValueError(f"Input [{name}] does not exist")

        return (
            environ[environ_input_key]
            if environ_input_key in environ
            else self._provided_inputs[environ_provided_key]
        )

    @staticmethod
    def _environ_key(name: str) -> str:
        return f"INPUT_{name.upper()}"

    @staticmethod
    def _environ_provided_key(name: str) -> str:
        return f"GITHUB_{name.upper()}"

    @staticmethod
    def _sync_action_file(personal_access_token: str = ""):
        """
        internal method for syncing the action file
        """
        content = GithubAPI.get_public_file(
            "stephend017", "ghapd", "action.yml", personal_access_token
        )
        with open("./action.yml", "w") as fp:
            fp.write(content)
        time.sleep(1)



from sd_utils.plugins.plugin_collector import collect_plugins
from pencil_pusher.configs.base import ConfigParamBase
from pencil_pusher.configs.manager import ConfigManager


config_manager = ConfigManager()
"""
Global config manager (needed for the config register decorator it provides)
"""

__all__ = collect_plugins(
    __file__, __name__, ConfigParamBase, ["base", "manager"]
)
"""
Loads all the plugins and makes them accessible throughout the lifespan
of the program
"""



import subprocess
from typing import List


class ProcessUtil:
    """
    """

    @staticmethod
    def execute(
        command: List[str], cwd: str = "./", stdout: int = subprocess.PIPE
    ) -> int:
        """
        runs a given command to completion using subprocess then
        returns the return code

        Args:
            command (List[str]): the command to run, where each element in the list
                is either a command or its arguments
            cwd (str): the directory to run this command in
            stdout (int): the file to write the output to
        """
        process = subprocess.Popen(
            command, cwd=cwd, stdout=stdout, stderr=subprocess.PIPE
        )
        process.communicate()
        return process.returncode



from typing import Any
from pencil_pusher.configs.manager import ConfigManager
from pencil_pusher.configs.base import ConfigParamBase
from pencil_pusher.configs import config_manager


@config_manager.register("sources")
class SourcesConfigParam(ConfigParamBase):
    """
    Processes the sources config parameter

    Note: validation only validates config name
        and config type (string array)
    """

    def on_iterate(self, data: Any):
        """
        This is the validate function
        """
        if data["type"] == ConfigManager.VALIDATE:
            self._validate(data["contents"])
            self.set_defined()

    def _validate(self, contents: dict) -> bool:
        """
        Validates the parameter is included in the
        config file since it is a required field
        """
        if "sources" not in contents:
            raise ValueError("sources config parameter was not defined")

        if not isinstance(contents["sources"], list):
            raise ValueError("source config parameter is not of type list")

        for source in contents["sources"]:
            if not isinstance(source, str):
                raise ValueError("sources path must be of type string")

            if "~" in source:
                raise ValueError(
                    "Illegal character [~] in source location. All source locations must be relative"
                )

        return True



import json
from typing import List
import pytest
from pencil_pusher.configs import config_manager


def load_config(
    sources=["path1/*", "/path1/subpath1/", "/path1/subpath1/file1.py"],
    title_prefix="valid_prefix",
    title_suffix="valid_suffix",
    titles=[{"source": "mypath/mysourcefile.py", "title": "mynewtitle"}],
    python={
        "include_init_files": False,
        "include_main_file": True,
        "module": "mymod",
    },
    exclude: List[str] = [],
    sidebar: bool = False,
):
    """
    Loads a config into the test config file,
    used to setup config tests.

    No parameters are typed so invalid types can
    be simulated. Instead a default value of the
    correct type is provided for each value
    """

    with open("./tests/test_configs/test.config.json", "w") as fp:
        data = {
            "sources": sources,
            "title_prefix": title_prefix,
            "title_suffix": title_suffix,
            "titles": titles,
            "python": python,
            "sidebar": sidebar,
        }
        for element in exclude:
            del data[element]
        json.dump(data, fp)


def load_and_validate():
    """
    Loads and validates the test config file
    """
    config_manager.load_config_file("./tests/test_configs/test.config.json")
    config_manager.validate()


def load_and_validate_fails():
    """
    Loads and validates the test config file that
    contains errors (is invalid)
    """
    config_manager.load_config_file("./tests/test_configs/test.config.json")
    with pytest.raises(ValueError):
        config_manager.validate()



from typing import Dict, List, Set
from pencil_pusher.process_util import ProcessUtil


class Documenter:
    @staticmethod
    def install():
        """
        installs the correct pydoc-markdown package
        to the python version being used to execute
        the commands
        """
        ProcessUtil.execute(
            ["python3", "-m", "pip", "install", "pydoc-markdown"]
        )

    @staticmethod
    def generate(module: str, title: str = "", output_dir: str = "./"):
        """
        Generates markdown documentation for a given
        module (.py file) and creates a file with the
        name title as the documentation output

        Args:
            module (str): the name of the module the file is being
                imported from
            title (str): the title of the document file to output
            output_dir (str): the directory to write the output file to
        """
        options = '{renderer: {type: markdown,descriptive_class_title: false,render_toc: true, header_level_by_type: {"Module": 1,"Class": 2,"Method": 3,"Function": 2,"Data": 2}}}'
        if not output_dir.endswith("/"):
            output_dir += "/"

        fp = open(f"{output_dir}{title}.md", "w")
        ProcessUtil.execute(
            ["pydoc-markdown", "-m", f"{module}", options], stdout=fp
        )
        fp.close()

    @staticmethod
    def build_toc(file_list: List[str]) -> Dict[str, str]:
        """
        Builds the table of contents from a file list
        assumes that the files are in python import notation

        Args:
            file_list (List[str]): the list of files to include
                the in the toc

        Returns:
            Dict[str, str]: the table of contents ordered as a
                dictionary

        """
        hierarchy = {}
        for file_path in file_list:
            parts = file_path.split(".")

            sub_heirarchy = hierarchy

            # exclude file extension
            for part in parts[:-1]:
                if part not in sub_heirarchy:
                    sub_heirarchy[part] = {}
                sub_heirarchy = sub_heirarchy[part]

        return hierarchy

    @staticmethod
    def generate_sidebar(
        toc: Dict[str, str], file_map: Dict[str, str], output_dir: str = "./"
    ):
        """

        The Sidebar takes the following format

        -- user-defined toc (not implemented)

        -- References
        -- custom TOC
        """
        fp = open(f"{output_dir}_Sidebar.md", "w")
        Documenter.sidebar_helper("", toc, file_map, fp, output_dir)

        fp.close()

    @staticmethod
    def sidebar_helper(
        prefix: str,
        toc: Dict[str, str],
        file_map: Dict[str, str],
        fp,
        output_dir: str = "./",
        level: int = 0,
    ):
        """
        """
        included = []
        for entry, value in toc.items():
            path = entry if prefix == "" else f"{prefix}.{entry}"
            if value == {}:
                # no sub entries, write to TOC
                fp.write(
                    Documenter.toc_entry(
                        entry, f"{output_dir}{file_map[path]}.md", level
                    )
                    + "\n"
                )
            else:
                # TODO make this generate a header page
                if entry not in included:
                    included.append(entry)
                    fp.write(Documenter.toc_header(entry, level) + "\n")
                Documenter.sidebar_helper(
                    path, value, file_map, fp, output_dir, level + 1
                )

    @staticmethod
    def toc_entry(title: str, path: str, level: int = 0):
        """
        """
        initial_spacing = " " * level * 2

        return f"{initial_spacing}- **[{title}]({path})**"

    @staticmethod
    def toc_header(title: str, level: int = 0):
        """
        """
        initial_spacing = " " * level * 2

        # TODO add header page link
        return f"{initial_spacing}- **[{title}]()**"



from pencil_pusher.documenter import Documenter
from pencil_pusher.input import InputManager
from pencil_pusher.github_api import GithubAPI
from pencil_pusher.repo_manager import RepoManager
from pencil_pusher.configs import config_manager


def main():
    im = InputManager()
    im.define()
    response = im.validate()

    if not response[0]:
        raise ValueError(response[1])

    Documenter.install()

    gh = GithubAPI(im.get("github_token"))

    owner_name = im.get("owner_name")
    repository_name = im.get("repository_name")

    rm = RepoManager(owner_name, repository_name, gh)
    rm.setup()
    rm.install()

    # load configs for the given file
    config_manager.load_config_file(f'{rm._repo_path}/{im.get("config_file")}')
    config_manager.validate()

    langs = config_manager.get_langs()
    # extensions = [ext for ext in [ext_list for _, ext_list in langs.items()]]
    extensions = []
    for _, ext_list in langs.items():
        for ext in ext_list:
            extensions.append(ext)

    # transform titles config into usable dict
    titles = {}
    for element in config_manager.get("titles"):
        has_defined_ext = False
        for ext in extensions:
            if element["source"].endswith(f".{ext}"):
                has_defined_ext = True
                titles[element["source"][: -(len(ext) + 1)]] = element["title"]
        if not has_defined_ext:
            titles[element["source"]] = element["title"]

    # only support default for now (repo_name = package_name)
    rm.document(
        sources=config_manager.get("sources"),
        module=""
        if not config_manager.has("python")
        else config_manager.get("python")["module"],
        title_prefix=config_manager.get("title_prefix"),
        title_suffix=config_manager.get("title_suffix"),
        titles=titles,
        extensions=extensions,
        sidebar=config_manager.get("sidebar"),
    )

    rm.publish()
    rm.cleanup()


if __name__ == "__main__":
    main()



from unittest import mock
from pencil_pusher.input import InputManager
import os


@mock.patch(
    "pencil_pusher.input.environ",
    {
        "GITHUB_REPOSITORY": "o_n/r_n",
        "INPUT_GITHUB_TOKEN": "gh_t",
        "INPUT_CONFIG_FILE": "c_f",
    },
)
def test_input_manager():
    """
    Validates that this method is working correctly
    """
    im = InputManager()
    im.define(os.environ["GH_PAT"])

    assert im.validate()

    # these are provided by the github action
    assert im.get("owner_name") == "o_n"
    assert im.get("repository_name") == "r_n"

    assert im.get("github_token") == "gh_t"
    assert im.get("config_file") == "c_f"



import os
from typing import List


class FileUtil:
    @staticmethod
    def query_directory(
        directory: str, include_extensions: List[str]
    ) -> List[str]:
        """
        Returns a list of file paths in the given directory and any
        of its containing sub directories

        Args:
            directory (str): the directory to query
            include_extensions (List[str]): a list of file extensions
                that should be included in the result set. any files
                with extensions not defined here will be ignored.
                Note: this should only inlcude the extension name
                and not a prefixed '.'
        """
        result: List[str] = []

        path = os.path.abspath(directory)

        for item in os.listdir(path):
            for ext in include_extensions:
                if item.endswith(ext):
                    result.append(path + "/" + item)
                    break
            if os.path.isdir(path + "/" + item):
                result.extend(
                    FileUtil.query_directory(
                        path + "/" + item + "/", include_extensions
                    )
                )

        return result



from pencil_pusher.configs.base import ConfigParamBase
from pencil_pusher.configs import config_manager


@config_manager.register("titles")
class TitlesConfigParam(ConfigParamBase):
    """
    Processes the titles config parameter

    Note: validation only validates config name
        and config type (string array)
    """

    _param_subtype = {"source": (str, True), "title": (str, True)}

    def _validate(self, contents: dict) -> bool:
        """
        Validates the parameter is included in the
        config file since it is a required field
        """
        if "titles" not in contents:
            # not required
            return True

        if not isinstance(contents["titles"], list):
            raise ValueError("titles config parameter is not of type list")

        for element in contents["titles"]:
            if not isinstance(element, dict):
                raise ValueError("titles config entry must be of type dict")

            for key, value in self._param_subtype.items():
                if key not in element:
                    if value[1]:
                        raise ValueError(
                            f"titles config key [{key}] is required but was not defined"
                        )
                    continue
                if not isinstance(element[key], value[0]):
                    raise ValueError(
                        f"titles config key [{key}] is invalid type, expected [{value[0]}] but found [{type(element[key])}]"
                    )

        return True



from tests.test_configs.config_helper import (
    load_and_validate,
    load_and_validate_fails,
    load_config,
)
from pencil_pusher.configs import config_manager


def test_valid():
    """
    Tests that a valid config file does not throw
    any errors when validated
    """
    load_config()
    load_and_validate()


def test_invalid_type():
    """
    tests that a config file with an invalid type
    for sources throws an error
    """
    load_config(sources="wrong")
    load_and_validate_fails()


def test_invalid_element_type():
    """
    tests that a complex type with an invalid
    element type throws an error
    """
    load_config(sources=["path1/", 2])
    load_and_validate_fails()


def test_invalid_name():
    """
    tests that a missing config parameter that is required
    throws an error
    """
    load_config(exclude=["sources"])
    load_and_validate_fails()


def test_get_values():
    """
    tests that config values are loaded properly
    """
    expected = [
        "path1/*",
        "path1/anotherpath/*",
        "path1/anotherpath2/",
        "path1/anotherpath2/file1.py",
    ]

    load_config(sources=expected)
    load_and_validate()

    response = config_manager.get("sources")
    assert len(expected) == len(response)
    for path in expected:
        assert path in response


def test_tilde_in_source():
    """
    Tests that a source does not contain any non relative paths
    i.e. tilde (~)
    """
    load_config(sources=["path1/tets", "~/path2/wrong.py"])
    load_and_validate_fails()



"""
This module contains functions for managing a repository

This includes
- cloning a repo
- cloning a repo's wiki
- installing a repo's python project

"""
import shutil
from subprocess import check_output
from typing import List
from pencil_pusher.process_util import ProcessUtil
from pencil_pusher.file_util import FileUtil
from pencil_pusher.documenter import Documenter
from pencil_pusher.github_api import GithubAPI
from pencil_pusher.configs import config_manager


class RepoManager:
    def __init__(self, owner: str, repo: str, github_api: GithubAPI):
        """
        Creates a RepoManager object, designed to manage a repo and
        its corresponding wiki.

        Args:
            owner (str): the owner of the repo
            repo (str): the name of the repo
            github_api (GithubAPI): the `GithubAPI` object being
                used to iterface this repo with github
        """
        self._owner = owner
        self._repo = repo
        self._gh = github_api

    def setup(self):
        """
        clones both this repo and its wiki into their
        own directories
        """
        self._gh.clone_repo(self._owner, self._repo, self._repo_path)
        self._gh.clone_repo(self._owner, f"{self._repo}.wiki", self._wiki_path)

    def install(self):
        """
        installs the python project in the source repo
        using the command pip install . (from source repo)
        """
        ProcessUtil.execute(
            ["python3", "-m", "pip", "install", "."], cwd=self._repo_path
        )

    def document(
        self,
        sources: List[str] = [],
        module: str = "",
        title_prefix: str = "",
        title_suffix: str = "",
        titles: dict = {},
        extensions: List[str] = [],
        sidebar: bool = False,
    ):
        """
        documents all given modules in the source
        repo (using config definition) then outputs
        them to the wiki repo and commits them

        Args:
            sources (List[str]): the sources defined by the config file to document
            module (str): the name of the base python module in the repo (defaults
                to the same name as the repo)
            title_prefix (str): text to append before the title of a generated file
            title_suffix (str): text to append after the title of a generated file
            extensions (List[str]): any extension (excluding leading '.') to check for
                in the sources (only applies to directory sources)
        """
        if module == "":
            # use default module name (same as repo name)
            module = self._repo

        file_list = self._get_file_list(sources, extensions)
        file_map = {}

        for f in file_list:
            module_python_path = f.replace("/", ".")
            title = f

            # use user defined titles if they exist
            if title in titles:
                title = titles[title]
            else:
                title = module_python_path

            file_map[module_python_path] = title_prefix + title + title_suffix

            Documenter.generate(
                module_python_path,
                title_prefix + title + title_suffix,
                self._wiki_path,
            )

        if sidebar:
            toc = Documenter.build_toc(file_map.keys())
            Documenter.generate_sidebar(
                toc, file_map, output_dir=self._wiki_path
            )

    def publish(self):
        """
        Publishes the generated documentation to the
        wiki repo
        """
        self._gh.full_update(
            self._owner, f"{self._repo}.wiki", self._wiki_path
        )

    def cleanup(self):
        """
        removes all files created by this object
        """
        shutil.rmtree(self._repo_path)
        shutil.rmtree(self._wiki_path)

    @property
    def _repo_path(self) -> str:
        """
        Returns the relative path of the source repo
        """
        return f"../{self._repo}"

    @property
    def _wiki_path(self) -> str:
        """
        Returns the relative path of the wiki repo
        """
        return f"../{self._repo}-wiki"

    def _get_file_list(
        self, sources: List[str], extensions: List[str]
    ) -> List[str]:
        """
        Gets a list of all files to process

        Args:
            sources (List[str]): the user defined sources to process
            extensions (List[str]): the file extensions to check for
                (only applies to directory sources)

        Returns:
            List[str]: a list of file paths to process for documentation
        """
        file_list = []

        for source in sources:
            if source.endswith("/"):
                # source is a directory
                directory_files = FileUtil.query_directory(
                    f"../{self._repo}/{source[:-1]}", extensions
                )
                for sf in directory_files:
                    response = config_manager.process_lang_file(sf)
                    if not response[0]:
                        continue
                    relative_source_file = sf[
                        sf.index(self._repo) + len(self._repo) + 1 : -3
                    ]
                    file_list.append(relative_source_file)

            else:
                # source is a file
                response = config_manager.process_lang_file(source)
                if not response[0]:
                    continue
                for ext in extensions:
                    if source.endswith(f".{ext}"):
                        # remove extension
                        source = source[: -(len(ext) + 1)]

                    file_list.append(source)
        return file_list



from setuptools import setup, find_packages


with open("requirements.txt", "r") as f:
    required = f.read().splitlines()

with open("README.md", "r") as f:
    readme = f.read()


setup(
    name="pencil_pusher",
    version="1.4.0",
    description="A python package that compiles source documentation and publishes it to the repo wiki",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Stephen Davis",
    author_email="stephenedavis17@gmail.com",
    packages=find_packages(),
    url="https://github.com/stephend017/pencil-pusher",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    install_requires=required,
)



from pencil_pusher.configs.manager import ConfigManager
from typing import Any
from sd_utils.plugins.plugin_base import PluginBase


class ConfigParamBase(PluginBase):
    """
    Base class for config parameters. All config parameters should
    extend this base class.
    """

    def __init__(self):
        self._is_defined = False

    def on_find(self, data: Any) -> Any:
        """
        Returns the value of this config

        Args:
            data (Any): A `dict` which contains the following values
                "type": a constant on what action is running
                "contents": the full config file as a dictionary
                "name": the name of the action being run

        Returns:
            Any: the value of this config
        """
        if not self._is_defined:
            return None
        return data["contents"][data["name"]]

    def on_iterate(self, data: Any):
        """
        This is the validate function

        Raises:
            ValueError: if the config is invalid. invalidity is
                determined by each config

        Args:
            data (Any): a `dict` with the following values
                "type": a constant on what action is being run (always VALIDATE)
                "contents": the full config file as a dictionary
        """
        if data["type"] == ConfigManager.VALIDATE:
            self._validate(data["contents"])
            self.set_defined()

    def _validate(self, contents: dict) -> bool:
        """
        method for validating this specific config given
        the entire config file as a dict

        Args:
            contents (dict): the full config file to validate
        """
        raise NotImplementedError

    def set_defined(self):
        """
        Sets this config parameter to having
        been defined.

        It is assumed that a config errors and
        is marked not valid if it is required
        but not defined
        """
        self._is_defined = True



from tests.test_configs.config_helper import (
    load_and_validate,
    load_and_validate_fails,
    load_config,
)
from pencil_pusher.configs import config_manager


def test_valid():
    """
    Tests that a valid config file contains the required
    parameter titles
    """
    load_config()
    load_and_validate()


def test_invalid_type():
    """
    """
    load_config(titles=3)
    load_and_validate_fails()


def test_invalid_subtype_source():
    """
    """
    load_config(titles=[{"source": 3, "title": "valid"}])
    load_and_validate_fails()


def test_invalid_subtype_title():
    """
    """
    load_config(titles=[{"source": "valid", "title": 2}])
    load_and_validate_fails()


def test_missing_subtype_source():
    """
    """
    load_config(titles=[{"title": "valid"}])
    load_and_validate_fails()


def test_missing_subtype_title():
    """
    """
    load_config(titles=[{"source": "valid"}])
    load_and_validate_fails()


def test_get_titles():
    """
    """
    expected = [{"source": "mysourcefile.py", "title": "wowsuchdocs"}]

    load_config(titles=expected)
    load_and_validate()

    response = config_manager.get("titles")
    assert response == expected



from typing import Any
from pencil_pusher.configs.manager import ConfigManager
from pencil_pusher.configs.base import ConfigParamBase
from pencil_pusher.configs import config_manager


@config_manager.register("python")
class PythonConfigParam(ConfigParamBase):
    """
    Processes the python config parameter

    Note: validation only validates config name
        and config type (string array)
    """

    _param_subtype = {
        # param name          type   requried   default
        "include_init_files": (bool, False, False),
        "include_main_file": (bool, False, False),
        "module": (str, False, ""),
    }

    def on_find(self, data: Any) -> Any:
        if data["type"] != ConfigManager.PROCESS_LANG_FILE:
            return super().on_find(data)

        if not data["contents"]["python"]["include_init_files"] and data[
            "file_path"
        ].endswith("__init__.py"):
            return (False, data["file_path"])
        if not data["contents"]["python"]["include_main_file"] and data[
            "file_path"
        ].endswith("__main__.py"):
            return (False, data["file_path"])
        return (True, data["file_path"])

    def on_register(self, data: Any):
        data["manager"].add_lang("python", ["py"])

    def on_iterate(self, data: Any):
        """
        This is the validate function
        """
        if data["type"] == ConfigManager.VALIDATE:
            self._validate(data["contents"])
            self.set_defined()

    def _validate(self, contents: dict) -> bool:
        """
        Validates the parameter is included in the
        config file since it is a required field
        """
        if "python" not in contents:
            contents["python"] = None
            return True

        if not isinstance(contents["python"], dict):
            raise ValueError("python config parameter is not of type dict")

        for key, value in self._param_subtype.items():
            if key not in contents["python"]:
                if value[1]:
                    raise ValueError(
                        f"python config key [{key}] is required but was not defined"
                    )
                contents["python"][key] = value[2]
                continue
            if not isinstance(contents["python"][key], value[0]):
                raise ValueError(
                    f"python config key [{key}] is invalid type, expected [{value[0]}] but found [{type(contents['python'][key])}]"
                )

        return True
